// $Id$
// Author: Yves Lafon <ylafon@w3.org>
//
// (c) COPYRIGHT MIT, ERCIM and Keio University, 2013.
// Please first read the full copyright statement in file COPYRIGHT.html
package org.w3c.css.properties.css;

import org.w3c.css.parser.CssSelectors;
import org.w3c.css.parser.CssStyle;
import org.w3c.css.properties.css2.Css2Style;
import org.w3c.css.util.ApplContext;
import org.w3c.css.util.InvalidParamException;
import org.w3c.css.values.CssExpression;
import org.w3c.css.values.CssValue;

/**
 * @since CSS2
 */
public class CssCue extends CssProperty {

	public CssValue value;

	public CssCueAfter cssCueAfter;
	public CssCueBefore cssCueBefore;

	/**
	 * Create a new CssCueAfter
	 */
	public CssCue() {
	}

	/**
	 * Creates a new CssCueAfter
	 *
	 * @param expression The expression for this property
	 * @throws org.w3c.css.util.InvalidParamException
	 *          Expressions are incorrect
	 */
	public CssCue(ApplContext ac, CssExpression expression, boolean check)
			throws InvalidParamException {
		throw new InvalidParamException("value",
				expression.getValue().toString(),
				getPropertyName(), ac);
	}

	public CssCue(ApplContext ac, CssExpression expression)
			throws InvalidParamException {
		this(ac, expression, false);
	}

	/**
	 * Returns the value of this property
	 */
	public Object get() {
		return value;
	}


	/**
	 * Returns the name of this property
	 */
	public final String getPropertyName() {
		return "cue";
	}

	/**
	 * Returns true if this property is "softly" inherited
	 * e.g. his value is equals to inherit
	 */
	public boolean isSoftlyInherited() {
		return value.equals(inherit);
	}

	/**
	 * Returns a string representation of the object.
	 */
	public String toString() {
		return value.toString();
	}

	/**
	 * Add this property to the CssStyle.
	 *
	 * @param style The CssStyle
	 */
	public void addToStyle(ApplContext ac, CssStyle style) {
		if (((Css2Style) style).cssCue != null)
			style.addRedefinitionWarning(ac, this);
		((Css2Style) style).cssCue = this;
		cssCueAfter.addToStyle(ac, style);
		cssCueBefore.addToStyle(ac, style);
	}

	/**
	 * Compares two properties for equality.
	 *
	 * @param property The other property.
	 */
	public boolean equals(CssProperty property) {
		return (property instanceof CssCue &&
				value.equals(((CssCue) property).value));
	}


	/**
	 * Get this property in the style.
	 *
	 * @param style   The style where the property is
	 * @param resolve if true, resolve the style to find this property
	 */
	public CssProperty getPropertyInStyle(CssStyle style, boolean resolve) {
		if (resolve) {
			return ((Css2Style) style).getCue();
		} else {
			return ((Css2Style) style).cssCue;
		}
	}

	/**
	 * Update the source file and the line.
	 * Overrides this method for a macro
	 *
	 * @param line   The line number where this property is defined
	 * @param source The source file where this property is defined
	 */
	public void setInfo(int line, String source) {
		super.setInfo(line, source);
		cssCueAfter.setInfo(line, source);
		cssCueBefore.setInfo(line, source);
	}

	/**
	 * Set this property to be important.
	 * Overrides this method for a macro
	 */
	public void setImportant() {
		super.setImportant();
		cssCueAfter.setImportant();
		cssCueBefore.setImportant();
	}

	/**
	 * Set the context.
	 * Overrides this method for a macro
	 *
	 * @see org.w3c.css.css.CssCascadingOrder#order
	 * @see org.w3c.css.css.StyleSheetParser#handleRule
	 */
	public void setSelectors(CssSelectors selector) {
		super.setSelectors(selector);
		cssCueAfter.setSelectors(selector);
		cssCueBefore.setSelectors(selector);
	}
}

